@@ -3,15 +3,15 @@ use super::condition_sanitizers::{
     parse_amount, parse_height, parse_seconds, sanitize_announce_msg, sanitize_hash,
 };
 use super::opcodes::{
-    parse_opcode, ConditionOpcode, AGG_SIG_COST, AGG_SIG_ME, AGG_SIG_UNSAFE,
+    parse_opcode, ConditionOpcode, AGG_SIG_COST, AGG_SIG_ME, AGG_SIG_UNSAFE, ALWAYS_TRUE,
     ASSERT_COIN_ANNOUNCEMENT, ASSERT_HEIGHT_ABSOLUTE, ASSERT_HEIGHT_RELATIVE, ASSERT_MY_AMOUNT,
     ASSERT_MY_COIN_ID, ASSERT_MY_PARENT_ID, ASSERT_MY_PUZZLEHASH, ASSERT_PUZZLE_ANNOUNCEMENT,
     ASSERT_SECONDS_ABSOLUTE, ASSERT_SECONDS_RELATIVE, CREATE_COIN, CREATE_COIN_ANNOUNCEMENT,
     CREATE_COIN_COST, CREATE_PUZZLE_ANNOUNCEMENT, RESERVE_FEE,
 };
-use super::rangeset::RangeSet;
 use super::sanitize_int::sanitize_uint;
 use super::validation_error::{first, next, rest, ErrorCode, ValidationErr};
+use crate::bytes::Bytes32;
 use crate::gen::flags::COND_ARGS_NIL;
 use crate::gen::flags::NO_UNKNOWN_CONDS;
 use crate::gen::flags::STRICT_ARGS_COUNT;
@@ -19,7 +19,7 @@ use crate::gen::validation_error::check_nil;
 use clvmr::allocator::{Allocator, NodePtr, SExp};
 use clvmr::cost::Cost;
 use clvmr::op_utils::u64_from_bytes;
-use clvmr::sha2::Sha256;
+use clvmr::sha2::{Digest, Sha256};
 use std::cmp::max;
 use std::collections::HashSet;
 use std::hash::{Hash, Hasher};
@@ -86,11 +86,10 @@ pub enum Condition {
     Skip,
 }
 
-fn parse_args(
+pub fn parse_args(
     a: &Allocator,
     mut c: NodePtr,
     op: ConditionOpcode,
-    range_cache: &mut RangeSet,
     flags: u32,
 ) -> Result<Condition, ValidationErr> {
     match op {
@@ -133,13 +132,7 @@ fn parse_args(
         CREATE_COIN => {
             let puzzle_hash = sanitize_hash(a, first(a, c)?, 32, ErrorCode::InvalidPuzzleHash)?;
             c = rest(a, c)?;
-            let amount = parse_amount(
-                a,
-                first(a, c)?,
-                ErrorCode::InvalidCoinAmount,
-                range_cache,
-                flags,
-            )?;
+            let amount = parse_amount(a, first(a, c)?, ErrorCode::InvalidCoinAmount)?;
             // CREATE_COIN takes an optional 3rd parameter, which is a list of
             // byte buffers (typically a 32 byte hash). We only pull out the
             // first element for now, but may support more in the future.
@@ -171,13 +164,7 @@ fn parse_args(
             Ok(Condition::CreateCoin(puzzle_hash, amount, a.null()))
         }
         RESERVE_FEE => {
-            let fee = parse_amount(
-                a,
-                first(a, c)?,
-                ErrorCode::ReserveFeeConditionFailed,
-                range_cache,
-                flags,
-            )?;
+            let fee = parse_amount(a, first(a, c)?, ErrorCode::ReserveFeeConditionFailed)?;
             if (flags & STRICT_ARGS_COUNT) != 0 {
                 check_nil(a, rest(a, c)?)?;
             }
@@ -238,13 +225,7 @@ fn parse_args(
             Ok(Condition::AssertMyPuzzlehash(id))
         }
         ASSERT_MY_AMOUNT => {
-            let amount = parse_amount(
-                a,
-                first(a, c)?,
-                ErrorCode::AssertMyAmountFailed,
-                range_cache,
-                flags,
-            )?;
+            let amount = parse_amount(a, first(a, c)?, ErrorCode::AssertMyAmountFailed)?;
             if (flags & STRICT_ARGS_COUNT) != 0 {
                 check_nil(a, rest(a, c)?)?;
             }
@@ -258,8 +239,6 @@ fn parse_args(
                 a,
                 first(a, c)?,
                 ErrorCode::AssertSecondsRelative,
-                range_cache,
-                flags,
             )?))
         }
         ASSERT_SECONDS_ABSOLUTE => {
@@ -270,22 +249,13 @@ fn parse_args(
                 a,
                 first(a, c)?,
                 ErrorCode::AssertSecondsAbsolute,
-                range_cache,
-                flags,
             )?))
         }
         ASSERT_HEIGHT_RELATIVE => {
             if (flags & STRICT_ARGS_COUNT) != 0 {
                 check_nil(a, rest(a, c)?)?;
             }
-            match sanitize_uint(
-                a,
-                first(a, c)?,
-                4,
-                ErrorCode::AssertHeightRelative,
-                range_cache,
-                flags,
-            ) {
+            match sanitize_uint(a, first(a, c)?, 4, ErrorCode::AssertHeightRelative) {
                 // Height is always positive, so a negative requirement is always true,
                 Err(ValidationErr(_, ErrorCode::NegativeAmount)) => Ok(Condition::Skip),
                 Err(r) => Err(r),
@@ -300,17 +270,19 @@ fn parse_args(
                 a,
                 first(a, c)?,
                 ErrorCode::AssertHeightAbsolute,
-                range_cache,
-                flags,
             )?))
         }
+        ALWAYS_TRUE => {
+            // this condition is always true, we always ignore arguments
+            Ok(Condition::Skip)
+        }
         _ => Err(ValidationErr(c, ErrorCode::InvalidConditionOpcode)),
     }
 }
 
 #[derive(Debug)]
 pub struct NewCoin {
-    pub puzzle_hash: Vec<u8>,
+    pub puzzle_hash: Bytes32,
     pub amount: u64,
     // the hint is optional. When not provided, this points to null (NodePtr
     // value -1). The hint is not part of the unique identity of a coin, it's not
@@ -336,7 +308,7 @@ impl PartialEq for NewCoin {
 // These are all the conditions related directly to a specific spend.
 #[derive(Debug)]
 pub struct Spend {
-    pub coin_id: Arc<[u8; 32]>,
+    pub coin_id: Arc<Bytes32>,
     pub puzzle_hash: NodePtr,
     // conditions
     // all these integers are initialized to 0, which also means "no
@@ -379,7 +351,7 @@ pub struct SpendBundleConditions {
 struct ParseState {
     // hashing of the announcements is deferred until parsing is complete. This
     // means less work up-front, in case parsing/validation fails
-    announce_coin: HashSet<(Arc<[u8; 32]>, NodePtr)>,
+    announce_coin: HashSet<(Arc<Bytes32>, NodePtr)>,
     announce_puzzle: HashSet<(NodePtr, NodePtr)>,
 
     // the assert announcements are checked once everything has been parsed and
@@ -390,11 +362,7 @@ struct ParseState {
     // all coin IDs that have been spent so far. When we parse a spend we also
     // compute the coin ID, and stick it in this set. It's reference counted
     // since it may also be referenced by announcements
-    spent_coins: HashSet<Arc<[u8; 32]>>,
-
-    // this object tracks which ranges of the heap we've scanned for zeros, to
-    // avoid scanning the same ranges multiple times.
-    range_cache: RangeSet,
+    spent_coins: HashSet<Arc<Bytes32>>,
 }
 
 impl ParseState {
@@ -405,7 +373,6 @@ impl ParseState {
             assert_coin: HashSet::new(),
             assert_puzzle: HashSet::new(),
             spent_coins: HashSet::new(),
-            range_cache: RangeSet::new(),
         }
     }
 }
@@ -422,15 +389,7 @@ fn parse_spend_conditions(
     spend = rest(a, spend)?;
     let puzzle_hash = sanitize_hash(a, first(a, spend)?, 32, ErrorCode::InvalidPuzzleHash)?;
     spend = rest(a, spend)?;
-    let amount_buf = sanitize_uint(
-        a,
-        first(a, spend)?,
-        8,
-        ErrorCode::InvalidCoinAmount,
-        &mut state.range_cache,
-        flags,
-    )?
-    .to_vec();
+    let amount_buf = sanitize_uint(a, first(a, spend)?, 8, ErrorCode::InvalidCoinAmount)?.to_vec();
     let my_amount = u64_from_bytes(&amount_buf);
     let cond = rest(a, spend)?;
     let coin_id = Arc::new(compute_coin_id(a, parent_id, puzzle_hash, &amount_buf));
@@ -483,7 +442,7 @@ fn parse_spend_conditions(
             _ => (),
         }
         c = rest(a, c)?;
-        let cva = parse_args(a, c, op, &mut state.range_cache, flags)?;
+        let cva = parse_args(a, c, op, flags)?;
         match cva {
             Condition::ReserveFee(limit) => {
                 // reserve fees are accumulated
@@ -494,7 +453,7 @@ fn parse_spend_conditions(
             }
             Condition::CreateCoin(ph, amount, hint) => {
                 let new_coin = NewCoin {
-                    puzzle_hash: a.atom(ph).to_vec(),
+                    puzzle_hash: a.atom(ph).into(),
                     amount,
                     hint,
                 };
@@ -519,7 +478,7 @@ fn parse_spend_conditions(
                 ret.height_absolute = max(ret.height_absolute, h);
             }
             Condition::AssertMyCoinId(id) => {
-                if a.atom(id) != *spend.coin_id {
+                if a.atom(id) != (*spend.coin_id).as_ref() {
                     return Err(ValidationErr(c, ErrorCode::AssertMyCoinIdFailed));
                 }
             }
@@ -600,17 +559,17 @@ pub fn parse_spends(
     // check all the assert announcements
     // if there are no asserts, there is no need to hash all the announcements
     if !state.assert_coin.is_empty() {
-        let mut announcements = HashSet::<[u8; 32]>::new();
+        let mut announcements = HashSet::<Bytes32>::new();
 
         for (coin_id, announce) in state.announce_coin {
             let mut hasher = Sha256::new();
-            hasher.update(&*coin_id);
+            hasher.update(*coin_id);
             hasher.update(a.atom(announce));
-            announcements.insert(hasher.finish());
+            announcements.insert(hasher.finalize().as_slice().into());
         }
 
         for coin_assert in state.assert_coin {
-            if !announcements.contains(a.atom(coin_assert)) {
+            if !announcements.contains(&a.atom(coin_assert).into()) {
                 return Err(ValidationErr(
                     coin_assert,
                     ErrorCode::AssertCoinAnnouncementFailed,
@@ -620,17 +579,17 @@ pub fn parse_spends(
     }
 
     if !state.assert_puzzle.is_empty() {
-        let mut announcements = HashSet::<[u8; 32]>::new();
+        let mut announcements = HashSet::<Bytes32>::new();
 
         for (puzzle_hash, announce) in state.announce_puzzle {
             let mut hasher = Sha256::new();
             hasher.update(a.atom(puzzle_hash));
             hasher.update(a.atom(announce));
-            announcements.insert(hasher.finish());
+            announcements.insert(hasher.finalize().as_slice().into());
         }
 
         for puzzle_assert in state.assert_puzzle {
-            if !announcements.contains(a.atom(puzzle_assert)) {
+            if !announcements.contains(&a.atom(puzzle_assert).into()) {
                 return Err(ValidationErr(
                     puzzle_assert,
                     ErrorCode::AssertPuzzleAnnouncementFailed,
@@ -737,17 +696,17 @@ fn hash_buf(b1: &[u8], b2: &[u8]) -> Vec<u8> {
     let mut ctx = Sha256::new();
     ctx.update(b1);
     ctx.update(b2);
-    ctx.finish().to_vec()
+    ctx.finalize().to_vec()
 }
 
 #[cfg(test)]
-fn test_coin_id(parent_id: &[u8], puzzle_hash: &[u8], amount: u64) -> [u8; 32] {
+fn test_coin_id(parent_id: &[u8; 32], puzzle_hash: &[u8; 32], amount: u64) -> Bytes32 {
     let mut hasher = Sha256::new();
     hasher.update(parent_id);
     hasher.update(puzzle_hash);
     let buf = u64_to_bytes(amount);
     hasher.update(&buf);
-    hasher.finish()
+    hasher.finalize().as_slice().into()
 }
 
 // this is a very simple parser. It does not handle errors, because it's only
@@ -897,10 +856,7 @@ fn cond_test_cb(
 }
 
 #[cfg(test)]
-use crate::gen::flags::COND_CANON_INTS;
-
-#[cfg(test)]
-const MEMPOOL_MODE: u32 = COND_CANON_INTS | COND_ARGS_NIL | STRICT_ARGS_COUNT;
+const MEMPOOL_MODE: u32 = COND_ARGS_NIL | STRICT_ARGS_COUNT | NO_UNKNOWN_CONDS;
 
 #[cfg(test)]
 fn cond_test(input: &str) -> Result<(Allocator, SpendBundleConditions), ValidationErr> {
@@ -1706,14 +1662,13 @@ fn test_single_assert_my_amount_exceed_max() {
 #[test]
 fn test_single_assert_my_amount_overlong() {
     // ASSERT_MY_AMOUNT
-    // leading zeroes are ignored
-    let (a, conds) = cond_test_flag("((({h1} ({h2} (123 (((73 (0x0000007b )))))", 0).unwrap();
-
-    assert_eq!(conds.cost, 0);
-    assert_eq!(conds.spends.len(), 1);
-    let spend = &conds.spends[0];
-    assert_eq!(*spend.coin_id, test_coin_id(H1, H2, 123));
-    assert_eq!(a.atom(spend.puzzle_hash), H2);
+    // leading zeroes are disallowed
+    assert_eq!(
+        cond_test_flag("((({h1} ({h2} (123 (((73 (0x0000007b )))))", 0)
+            .unwrap_err()
+            .1,
+        ErrorCode::AssertMyAmountFailed
+    );
 }
 
 #[test]
@@ -1805,14 +1760,13 @@ fn test_single_assert_my_coin_id_extra_arg_mempool() {
 #[test]
 fn test_single_assert_my_coin_id_overlong() {
     // ASSERT_MY_COIN_ID
-    // leading zeros in the coin amount are ignored when computing the coin ID
-    let (a, conds) = cond_test_flag("((({h1} ({h2} (0x0000007b (((70 ({coin12} )))))", 0).unwrap();
-
-    assert_eq!(conds.cost, 0);
-    assert_eq!(conds.spends.len(), 1);
-    let spend = &conds.spends[0];
-    assert_eq!(*spend.coin_id, test_coin_id(H1, H2, 123));
-    assert_eq!(a.atom(spend.puzzle_hash), H2);
+    // leading zeros in the coin amount invalid
+    assert_eq!(
+        cond_test_flag("((({h1} ({h2} (0x0000007b (((70 ({coin12} )))))", 0)
+            .unwrap_err()
+            .1,
+        ErrorCode::InvalidCoinAmount
+    );
 }
 
 #[test]
@@ -2014,7 +1968,7 @@ fn test_single_create_coin() {
     assert_eq!(a.atom(spend.puzzle_hash), H2);
     assert_eq!(spend.create_coin.len(), 1);
     for c in &spend.create_coin {
-        assert_eq!(c.puzzle_hash, H2.to_vec());
+        assert_eq!(c.puzzle_hash, H2.into());
         assert_eq!(c.amount, 42_u64);
         assert_eq!(c.hint, a.null());
     }
@@ -2033,7 +1987,7 @@ fn test_create_coin_max_amount() {
     assert_eq!(a.atom(spend.puzzle_hash), H2);
     assert_eq!(spend.create_coin.len(), 1);
     for c in &spend.create_coin {
-        assert_eq!(c.puzzle_hash, H2.to_vec());
+        assert_eq!(c.puzzle_hash, H2.into());
         assert_eq!(c.amount, 0xffffffffffffffff_u64);
         assert_eq!(c.hint, a.null());
     }
@@ -2084,7 +2038,7 @@ fn test_create_coin_with_hint() {
     assert_eq!(a.atom(spend.puzzle_hash), H2);
     assert_eq!(spend.create_coin.len(), 1);
     for c in &spend.create_coin {
-        assert!(c.puzzle_hash == H2.to_vec());
+        assert!(c.puzzle_hash == H2.into());
         assert!(c.amount == 42_u64);
         assert!(a.atom(c.hint) == H1.to_vec());
     }
@@ -2104,7 +2058,7 @@ fn test_create_coin_extra_arg() {
     assert_eq!(a.atom(spend.puzzle_hash), H2);
     assert_eq!(spend.create_coin.len(), 1);
     for c in &spend.create_coin {
-        assert!(c.puzzle_hash == H2.to_vec());
+        assert!(c.puzzle_hash == H2.into());
         assert!(c.amount == 42_u64);
         assert!(a.atom(c.hint) == H1.to_vec());
     }
@@ -2137,7 +2091,7 @@ fn test_create_coin_with_multiple_hints() {
     assert_eq!(a.atom(spend.puzzle_hash), H2);
     assert_eq!(spend.create_coin.len(), 1);
     for c in &spend.create_coin {
-        assert!(c.puzzle_hash == H2.to_vec());
+        assert!(c.puzzle_hash == H2.into());
         assert!(c.amount == 42_u64);
         assert!(a.atom(c.hint) == H1.to_vec());
     }
@@ -2156,7 +2110,7 @@ fn test_create_coin_with_hint_as_atom() {
     assert_eq!(a.atom(spend.puzzle_hash), H2);
     assert_eq!(spend.create_coin.len(), 1);
     for c in &spend.create_coin {
-        assert_eq!(c.puzzle_hash, H2.to_vec());
+        assert_eq!(c.puzzle_hash, H2.into());
         assert_eq!(c.amount, 42_u64);
         assert_eq!(c.hint, a.null());
     }
@@ -2174,7 +2128,7 @@ fn test_create_coin_with_invalid_hint_as_terminator() {
     assert_eq!(a.atom(spend.puzzle_hash), H2);
     assert_eq!(spend.create_coin.len(), 1);
     for c in &spend.create_coin {
-        assert_eq!(c.puzzle_hash, H2.to_vec());
+        assert_eq!(c.puzzle_hash, H2.into());
         assert_eq!(c.amount, 42_u64);
         assert_eq!(c.hint, a.null());
     }
@@ -2205,7 +2159,7 @@ fn test_create_coin_with_short_hint() {
     assert_eq!(spend.create_coin.len(), 1);
 
     for c in &spend.create_coin {
-        assert!(c.puzzle_hash == H2.to_vec());
+        assert!(c.puzzle_hash == H2.into());
         assert!(c.amount == 42_u64);
         assert!(a.atom(c.hint) == MSG1.to_vec());
     }
@@ -2224,7 +2178,7 @@ fn test_create_coin_with_long_hint() {
     assert_eq!(spend.create_coin.len(), 1);
 
     for c in &spend.create_coin {
-        assert_eq!(c.puzzle_hash, H2.to_vec());
+        assert_eq!(c.puzzle_hash, H2.into());
         assert_eq!(c.amount, 42_u64);
         assert_eq!(c.hint, a.null());
     }
@@ -2244,7 +2198,7 @@ fn test_create_coin_with_pair_hint() {
     assert_eq!(spend.create_coin.len(), 1);
 
     for c in &spend.create_coin {
-        assert_eq!(c.puzzle_hash, H2.to_vec());
+        assert_eq!(c.puzzle_hash, H2.into());
         assert_eq!(c.amount, 42_u64);
         assert_eq!(a.atom(c.hint), H1.to_vec());
     }
@@ -2263,7 +2217,7 @@ fn test_create_coin_with_cons_hint() {
     assert_eq!(a.atom(spend.puzzle_hash), H2);
     assert_eq!(spend.create_coin.len(), 1);
     for c in &spend.create_coin {
-        assert_eq!(c.puzzle_hash, H2.to_vec());
+        assert_eq!(c.puzzle_hash, H2.into());
         assert_eq!(c.amount, 42_u64);
         assert_eq!(c.hint, a.null());
     }
@@ -2283,12 +2237,12 @@ fn test_multiple_create_coin() {
     assert_eq!(spend.create_coin.len(), 2);
 
     assert!(spend.create_coin.contains(&NewCoin {
-        puzzle_hash: H2.to_vec(),
+        puzzle_hash: H2.into(),
         amount: 42_u64,
         hint: a.null()
     }));
     assert!(spend.create_coin.contains(&NewCoin {
-        puzzle_hash: H2.to_vec(),
+        puzzle_hash: H2.into(),
         amount: 43_u64,
         hint: a.null()
     }));
@@ -2691,3 +2645,44 @@ fn test_double_spend() {
         ErrorCode::DoubleSpend
     );
 }
+
+#[test]
+fn test_always_true() {
+    // ALWAYS_TRUE
+    let (a, conds) = cond_test("((({h1} ({h2} (123 (((1 )))))").unwrap();
+
+    // just make sure there are no constraints
+    assert_eq!(conds.agg_sig_unsafe.len(), 0);
+    assert_eq!(conds.reserve_fee, 0);
+    assert_eq!(conds.height_absolute, 0);
+    assert_eq!(conds.seconds_absolute, 0);
+    assert_eq!(conds.cost, 0);
+
+    // there is one spend
+    assert_eq!(conds.spends.len(), 1);
+    let spend = &conds.spends[0];
+    assert_eq!(*spend.coin_id, test_coin_id(H1, H2, 123));
+    assert_eq!(a.atom(spend.puzzle_hash), H2);
+    assert_eq!(spend.agg_sig_me.len(), 0);
+}
+
+#[test]
+fn test_always_true_with_arg() {
+    // ALWAYS_TRUE, but with one unknown argument
+    // unknown arguments are expected and always allowed
+    let (a, conds) = cond_test("((({h1} ({h2} (123 (((1 ( 42 )))))").unwrap();
+
+    // just make sure there are no constraints
+    assert_eq!(conds.agg_sig_unsafe.len(), 0);
+    assert_eq!(conds.reserve_fee, 0);
+    assert_eq!(conds.height_absolute, 0);
+    assert_eq!(conds.seconds_absolute, 0);
+    assert_eq!(conds.cost, 0);
+
+    // there is one spend
+    assert_eq!(conds.spends.len(), 1);
+    let spend = &conds.spends[0];
+    assert_eq!(*spend.coin_id, test_coin_id(H1, H2, 123));
+    assert_eq!(a.atom(spend.puzzle_hash), H2);
+    assert_eq!(spend.agg_sig_me.len(), 0);
+}
